﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PojeClient {
    // ある程度消去せずに溜めて，一定の条件下で最大攻撃をする
    class AI_ChargeAndFire2 : Player {
        private int fMyIndex;
        private int fTimeLimit;
        private Action[] fAnswerBox;
        private Random fRand;

        public String getName() {
            return "AI : Charge and Fire II";
        }

        public void newGame(int yourIndex) {
            fRand = new Random(yourIndex+(new Random()).Next());
            fMyIndex = yourIndex;
        }

        public void endGame(int result) {
        }


        public void setParam(int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
        }

        public void makeAction(object currentState) {
            State state = (State)currentState;
            Board myBoard = state.getBoard(fMyIndex);
            Board oppBoard = state.getBoard(1 - fMyIndex);
            List<Action> possibleActions = myBoard.createPossibleActions();

            //// 危機が３手以内に迫っているなら，打てる中で最も攻撃力の高い手を打つ．
            int restTurn = int.MaxValue;
            // Case 1. 埋まりそうな場合．
            if (AITools.getRestCount(myBoard) < 17) restTurn = 3;
            if (AITools.getRestCount(myBoard) < 15) restTurn = 2;
            if (AITools.getRestCount(myBoard) < 13) restTurn = 1;
            // Case 2. 相手の攻撃がある程度あって近い場合．
            JamInfo hisAttack = AITools.calcOpponentsAttack(oppBoard, myBoard.getJamInfo(), state.getTurn());   // 今自分の上にある　＋　相手が連鎖中のもの　を含めた邪魔珠．
            hisAttack.deleteJam(oppBoard.getJamInfo().getTotalCount());                          // そこから，今相手の上にあるものを引く．これで実質攻撃される数と時間が分かる．

            if (hisAttack.getCountTill(state.getTurn() + 3) > 5) restTurn = 3;
            if (hisAttack.getCountTill(state.getTurn() + 2) > 5) restTurn = 2;
            if (hisAttack.getCountTill(state.getTurn() + 1) > 5) restTurn = 1;

            // 安全な場合は，適当にくっつくように消さないで置く　（Charge）
            if (restTurn == int.MaxValue) {
                int bestScore = int.MinValue;
                Action bestAction = null;
                foreach (Action a in possibleActions) {
                    int score;
                    AITool_Result result = AITools.calcTotalGain(myBoard, a, false);         // その手の攻撃結果
                    int connects = AITools.calcConnectCount(myBoard, a);  // その手がどれだけ周りの石と同色か

                    if (result.IsVanish == false) {
                        score = 70 * connects + fRand.Next(100);  //消去しない
                    } else {
                        score = -result.Gain;   //消去してしまうにしても小さいものがよい
                        if (result.Gain>100) score = result.Gain;   // （このクラスとしては）十分大きい攻撃ならしてしまう
                    }
                    if (score > bestScore) {
                        bestScore = score;
                        bestAction = a;
                    }
                }
                fAnswerBox[0] = bestAction;
                return;
            }

            // 危機が迫っている場合は，残りターン数の全ての置き方を読んで，最大攻撃力のものを返す (Fire)
            AITool_Result result2 = AITools.calcBestAttackAction(myBoard, possibleActions, restTurn);
            if (result2.Gain + result2.JamVanishCount>0) {
                fAnswerBox[0] = result2.Action;
            } else {
                // 消せないならランダムに置く．ほっとくと左端から積んでくので．
                fAnswerBox[0] = possibleActions[fRand.Next(possibleActions.Count)];
            }
        }
    }
}
