﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace PojeClient {
    // 描画関連の，概ね staticのもの
    class DrawTools {
        // 黒と，珠各種の色のブラシ
        public static Brush[] colorBrushes = new Brush[5] {new SolidBrush(Color.Black), new SolidBrush(Color.FromArgb(255,0,80)), new SolidBrush(Color.FromArgb(0,255,255)), new SolidBrush(Color.FromArgb(0,180,0)), new SolidBrush(Color.FromArgb(255,255,0)) };

        // 連鎖数表示用のフォント
        public static Font[] chainFonts = new Font[7] {new Font("Impact", 10), new Font("Impact", 11),new Font("Impact", 12), new Font("Impact", 13),new Font("Impact", 14), 
            new Font("Impact", 15),new Font("Impact", 16)};

        // 基準サイズ．もしかしたら将来大きさを変えられるようにするかもしれない．
        public const int stoneSize = 25;

        // 珠の表示．最初はEllipseでいいや
        public static void drawStone(Graphics g, int gx, int gy, int color) {
            // ない場合 
            if (color == 0) return;

            if (color == -1) {
                // 邪魔珠の場合
                g.DrawEllipse(Pens.Silver, gx+1, gy+1, stoneSize-2, stoneSize-2);
            } else {
                // 普通の珠の場合
                g.FillEllipse(colorBrushes[color], gx, gy, stoneSize, stoneSize);
            }
        }

        // 珠の表示の，XY座標版
        public static void drawStoneOnBoard(Graphics g, int x, int y, int color) {
            int gx = stoneSize * x;
            int gy = stoneSize * (5 + Settings.BoardHeight - y);
            drawStone(g, gx, gy, color);
        }

        // 連鎖数の表示
        public static void drawChainNumber(Graphics g, int x, int y, int color, int chain) {
            int fontIndex = color / 2;
            if (fontIndex >= chainFonts.Length) fontIndex = chainFonts.Length - 1;
            Font f = chainFonts[fontIndex];
            Brush b = colorBrushes[color];

            SizeF size = g.MeasureString(chain.ToString(), f);
            float left = stoneSize/2 + stoneSize * x - size.Width / 2;
            float top =  stoneSize/2 + stoneSize * (5+Settings.BoardHeight -y) - size.Height / 2;

            g.DrawString(chain.ToString(), f, b, new PointF(left, top));
        }
 
        // 消える邪魔珠の表示
        public static void drawVanishingJam(Graphics g, int x, int y) {
            int gx = stoneSize * x;
            int gy = stoneSize * (5 + Settings.BoardHeight - y);
            g.DrawEllipse(Pens.Gray, gx + 6, gy + 6, stoneSize - 12, stoneSize - 12);
        }



        // 落下予測位置の珠表示．
        public static void drawPredictedStone(Graphics g, int x, int y, int color) {
            int gx = stoneSize * x;
            int gy = stoneSize * (5 + Settings.BoardHeight - y);
            g.FillEllipse(colorBrushes[color], gx + 6, gy + 6, stoneSize - 12, stoneSize - 12);
        }


        // 珠の消去．
        public static void delStone(Graphics g, int gx, int gy) {
            g.FillRectangle(colorBrushes[0], gx+1, gy, stoneSize-1, stoneSize);
        }

        // 珠の消去．XY座標版
        public static void delStoneFromBoard(Graphics g, int x, int y) {
            int gx = stoneSize * x;
            int gy = stoneSize * (5 + Settings.BoardHeight - y);
            delStone(g, gx, gy);
        }


    }
}
