﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Drawing;

namespace PojeClient
{
    // １ゲームに関するほぼ全ての情報と権利を持ち，進行を行う．
    class GameManager {
        private BoardPainter[] fPainters;

        private Label fLb_Timer;
        private Label fLb_Turn;

        private Player[] fPlayers;
        private bool[] fIsUser;    // 左プレイヤが Player_Userかどうか．[1]はデフォルトfalse
        private bool[] fIsPTP;   // 右プレイヤが Player_PTP かどうか．[0]はデフォルトfalse
        
        private State fState;

        private Form_main fOwnerForm;
        private SGFManager fSGFManager;
        private VSServer fVSServer;

        public const int RESULT_DRAW = 0;
        public const int RESULT_WIN1 = 1;
        public const int RESULT_WIN2 = 2;
        public const int RESULT_STOPPED = 3;
        public static String[] RESULTSTRING = new String[4] { "Draw", "Player1 win", "Player2 win", "Stopped" };

        public const int PLAYTYPE_NORMAL = 0;
        public const int PLAYTYPE_REPLAY = 1;
        public const int PLAYTYPE_SERVER = 2;


        public GameManager(Form_main owner) {
            fOwnerForm = owner;
        }


        #region 登録関係

        public void setPicBoxes(PictureBox board1, PictureBox board2, PictureBox next1, PictureBox next2) {
            fPainters = new BoardPainter[2] { new BoardPainter(board1, next1), new BoardPainter(board2, next2) };
        }

        public void setLabels(Label lb_timer, Label lb_turn) {
            fLb_Turn = lb_turn;
            fLb_Timer = lb_timer;
        }

        public void setSGFManager(SGFManager manager) {
            fSGFManager = manager;
        }

        public void setVSServer(VSServer server) {
            fVSServer = server;
        }

        public void setPlayers(Player player1, Player player2) {
            fPlayers = new Player[2];
            fPlayers[0] = player1;
            fPlayers[1] = player2;
            
            fIsUser = new bool[2] { false, false };
            if (fPlayers[0].GetType() == typeof(Player_User)) {
                fIsUser[0] = true;
                ((Player_User)fPlayers[0]).setPainter(fPainters[0]);
            }

            fIsPTP = new bool[2] { false, false };
            if (fPlayers[1].GetType() == typeof(Player_PTP)) {
                fIsPTP[1] = true;
            }
        }

        #endregion


        // １ゲームを実施する．どちらかが潰れるか，投了するか，中止ボタンが押されるまで続ける． (playType=0)
        // 棋譜からの読み出しも同様にして行う (playType=1)
        // 将来多分，サーバ側もこれで実装する
        public int executeGame(int playType) {
            fState = new State();



            int limitTime=Settings.LimitTime;
            int[] firsts = null;
            if (playType == PLAYTYPE_SERVER) {
                // サーバ側の場合，最初の6珠の色をランダムに決定する．必ず２色は使うようにする．
                NextGenerator.initRand();
                firsts = NextGenerator.createFirstSet(Settings.NextOpenTurns * 2);
            } else if (playType == PLAYTYPE_REPLAY) {
                // リプレイの場合，棋譜から最初の６個の色を取り込む
                firsts = new int[Settings.NextOpenTurns*2];
                for (int i = 0; i < Settings.NextOpenTurns*2; i++) {
                    firsts[i] = fSGFManager.pickColors(0);
                    fSGFManager.pickColors(1);
                }
            } else if (fPlayers[1].GetType() == typeof(Player_PTP)) {
                // クライアント側の場合，接続を待ち，続いてサーバから最初の６個のnext珠をもらう
                if (((Player_PTP)fPlayers[1]).connect() == false) {
                    return RESULT_STOPPED;
                }
                firsts = ((Player_PTP)fPlayers[1]).waitUntilGetFirstStones(fPlayers[0].getName());
            } else {
                // 通常の場合，最初の6珠の色をランダムに決定する．必ず２色は使うようにする．
                NextGenerator.initRand();
                firsts = NextGenerator.createFirstSet(Settings.NextOpenTurns * 2);
            }

            fState.initialize(firsts);

            Board[] boards = new Board[2]{fState.getBoard(0), fState.getBoard(1)};

            // 描画する人へのボードの参照渡しと初期化
            for (int i = 0; i < 2; i++) {
                fPainters[i].setBoard(fState.getBoard(i));
                fPainters[i].showNext();
                fPainters[i].showBoard();
            }

            // Playerのゲーム前処理．リプレイやサーバの場合不要
            if (playType == PLAYTYPE_NORMAL) {
                fPlayers[0].newGame(0);
                fPlayers[1].newGame(1);
            }
            // 棋譜の準備．リプレイの場合不要．
            if (playType != PLAYTYPE_REPLAY) {
                fSGFManager.setFileName("Autosave.sgf");
                fSGFManager.setLimitTime(limitTime);
                fSGFManager.initialize(Settings.DrawTurn);

                if (playType == PLAYTYPE_NORMAL) {
                    fSGFManager.setPlayerNames(fPlayers[0].getName(), fPlayers[1].getName());
                } else if (playType == PLAYTYPE_SERVER) {
                    // ゲーム開始前のサーバの処理
                    // 名前を尋ねる
                    String s1 = fVSServer.askPlayerName(0);
                    Logger.log("Player1 : " + s1);
                    String s2 = fVSServer.askPlayerName(1);
                    Logger.log("Player2 : " + s2);
                    fSGFManager.setPlayerNames(s1, s2);

                    // 相手の名前を伝える
                    fVSServer.sendOpponentName(0, s2);
                    fVSServer.sendOpponentName(1, s1);

                    // ルール，持ち時間，配珠など，共通の情報を通知する
                    fVSServer.sendRules(firsts);
                }
            }

            Logger.log("Game Start");

            // 勝負のメインループ
            while (true) {
                Application.DoEvents();

                // Phase 1. ターン処理
                fState.incTurn();
                fLb_Turn.Text = "Turn = " + fState.getTurn();
                if (fState.getTurn() > Settings.DrawTurn) return endGame(playType, RESULT_DRAW);

                // Phase 2a. Jam落下
                for (int pl = 0; pl < 2; pl++) {
                    // 連鎖中のプレイヤは関係ない
                    if (boards[pl].getChainCount() > 0) continue; 

                    // 落ちる珠の数を取得
                    int count = boards[pl].getJamInfo().pickJamDropCount(fState.getTurn());

                    // 落とす
                    if (count > 0) {
                        boards[pl].transit0_JamDrop(count);
                        fPainters[pl].showBoard();
                        Thread.Sleep(Settings.TOTALWAIT * Settings.SLEEPAFTER_JAM);
                    }
                }

                // Phase2b. 死亡判定．Jamなしでも埋まって死ぬことはある．
                for (int pl = 0; pl < 2; pl++) {
                    if (boards[pl].getChainCount() > 0) continue;

                    if (boards[pl].isFilled() == true) return endGame(playType, 2-pl);
                }


                // サーバの場合，配珠色を送るための事前処理
                if (fState.getTurn() > 1 && playType == PLAYTYPE_SERVER) {
                    fVSServer.clearNexts();
                }

                // Phase 3. Next生成，両プレイヤの操作（や待機）
                // 途中の答えを保持する箱
                Action[][] answerActions = new Action[2][]{ new Action[1], new Action[1]};
                List<Thread> threads = new List<Thread>();
                for (int pl = 0; pl < 2; pl++) {
                    // 連鎖中のプレイヤは関係ない
                    if (boards[pl].getChainCount() > 0) {
                        answerActions[pl][0] = null;
                        continue;
                    }

                    // nextを生成する．初回はすでに生成されているので不要
                    if (fState.getTurn() > 1) {
                        if (playType == PLAYTYPE_SERVER) {
                            // サーバ側なら，連鎖の有無に合わせて色を作った上で，クライアントに配る
                            // クライアントに配る順番の関係上通信は後でまとめて行う
                            int[] nexts;
                            if (pl == 1 && fState.isNextSynchronized() == true) {
                                nexts=boards[0].getLastColors(2);
                            } else {
                                nexts= NextGenerator.createNextSet(boards[pl].getLastColors(2 * Settings.LimitTurnOfColorAbsent));
                            }
                            boards[pl].setNextColors(nexts);
                            fVSServer.prepareSendNexts(nexts);
                        } else if (playType == PLAYTYPE_REPLAY) {
                            // リプレイ中なら，棋譜から抜く
                            int[] cols = new int[2];
                            cols[0] = fSGFManager.pickColors(pl);
                            cols[1] = fSGFManager.pickColors(pl);
                            boards[pl].setNextColors(cols);
                        } else if (fPlayers[1].GetType() == typeof(Player_PTP)) {
                            // 通信対戦中なら，サーバからもらう
                            boards[pl].setNextColors(((Player_PTP)fPlayers[1]).waitUntilGetNexts());
                        } else if (pl == 1 && fState.isNextSynchronized() == true) {
                            // 通常プレイ中で連鎖がない状態なら，右側は左側と同じ色を来させる
                            boards[1].setNextColors(boards[0].getLastColors(2));
                        } else {
                            // 通常プレイ中なら，過去6個の色を渡して，次の2個の色を生成する
                            boards[pl].setNextColors(NextGenerator.createNextSet(boards[pl].getLastColors(2 * Settings.LimitTurnOfColorAbsent)));
                        }
                        // 描画する
                        fPainters[pl].showNext();
                    }

                    // デフォルトアクションを作り，表示する
                    Action act = new Action(boards[pl], boards[pl].getNextColors()[0], boards[pl].getNextColors()[1]);
                    boards[pl].setShownAction(null);
                    fPainters[pl].showAction(act);
                    answerActions[pl][0] = act;              // Playerから返事がない場合これをこのまま落とす．埋まってて落とせない場合は可能なところ[0]に落とす．

                    // 取りうる可能なActionの集合を作る
                    boards[pl].createPossibleActions();

                    // 再生中，サーバの場合は関係ない
                    if (playType == PLAYTYPE_NORMAL) {
                        // パラメータ渡し
                        fPlayers[pl].setParam(limitTime, answerActions[pl]);

                        // スレッド登録. UserやPTPの場合はしない
                        if (fIsUser[pl] == false && fIsPTP[pl] == false) threads.Add(new Thread(new ParameterizedThreadStart(fPlayers[pl].makeAction)));
                    }
                }

                // サーバの場合，配珠色をまとめて送る
                if (fState.getTurn() > 1 && playType == PLAYTYPE_SERVER) {
                    fVSServer.sendNexts();
                }


                // Stopボタンが押されていれば終わる
                if (fOwnerForm.getPushedKey() == Player_User.KEY_STOP) return endGame(playType, RESULT_STOPPED);

                // 行動を得る
                if (playType == PLAYTYPE_NORMAL) {
                    // まずコンピュータの思考を開始
                    foreach (Thread thread in threads) thread.Start(new State(fState));

                    // ユーザの行動が決まるまで待つ．この中で doEvents する
                    if (fIsUser[0] == true && boards[0].getChainCount() == 0) {
                        // タイマー起動
                        fOwnerForm.setTimerEnabled(true, limitTime);
                        // 行動決定待ち
                        fPlayers[0].makeAction(new State(fState));
                        // タイマー停止
                        fOwnerForm.setTimerEnabled(false, 0);

                        if (Signal.FormClosed==true) return RESULT_STOPPED;

                        if (answerActions[0][0].getValue() == Action.STOP) {
                            if (fIsPTP[1] == true) ((Player_PTP)fPlayers[1]).sendSpecialAction(Action.STOP);
                            return endGame(playType, RESULT_STOPPED);
                        }
                        if (answerActions[0][0].getValue() == Action.RESIGN) {
                            if (fIsPTP[1] == true) ((Player_PTP)fPlayers[1]).sendSpecialAction(Action.RESIGN);
                            return endGame(playType, RESULT_WIN2);
                        }
                    }

                    // コンピュータの思考終了, を待つ
                    DateTime startTime = DateTime.Now;
                    while (true) {
                        Thread.Sleep(10);
                        Application.DoEvents();
                        // case 1. フォームが閉じられれば終了（スレッドも強制終了すべきか？）
                        if (Signal.FormClosed == true) return RESULT_STOPPED;
                        // case 2. 全てのスレッドが終われば終了
                        bool allJoined = true;
                        foreach (Thread thread in threads) {
                            if (thread.IsAlive == true) allJoined = false;
                        }
                        if (allJoined == true) break;
                        // case 3. 制限時間が過ぎれば終了
                        DateTime now = DateTime.Now;
                        if ((now - startTime).TotalSeconds > limitTime) break;
                    }
                    if (Signal.FormClosed == true) return RESULT_STOPPED;

                    if (fIsPTP[1] == true) {
                        // PTPが相手の場合は，まず自分の思考が終わったらすぐ送り，続いて相手の手を待つ
                        // 打てないところの場合，先んじて修正する（Illegal負けにならないように）
                        // それぞれ，連鎖中の場合はしない
                        if (boards[0].getChainCount() == 0) {
                            if (answerActions[0][0].getValue() == Action.RESIGN) {
                                // 左がコンピュータで，投了した場合
                                ((Player_PTP)fPlayers[1]).sendSpecialAction(Action.RESIGN);
                                return endGame(playType, RESULT_WIN2);
                            } else {
                                // それが落下可能なのか確認し，無理なら別の場所[0]にする
                                if (answerActions[0][0].isLegal() == false) answerActions[0][0] = boards[0].getPossibleActions()[0];
                                // 通常の手をサーバに送る
                                ((Player_PTP)fPlayers[1]).sendAction(answerActions[0][0]);
                            }
                        }
                        if (boards[1].getChainCount() == 0) {
                            answerActions[1][0] = ((Player_PTP)fPlayers[1]).waitUntilOpponentAction(answerActions[1][0]);
                            // かえってきたのが STOP/RESIGN ならそこで終了
                            if (answerActions[1][0].getValue() == Action.STOP) return endGame(playType, RESULT_STOPPED);
                            if (answerActions[1][0].getValue() == Action.RESIGN) return endGame(playType, RESULT_WIN1);
                        }
                    } else {
                        // コンピュータが投了してくることもある
                        if (boards[0].getChainCount() == 0) {
                            if (answerActions[0][0].getValue() == Action.RESIGN) return endGame(playType, RESULT_WIN2);
                        }
                        if (boards[1].getChainCount() == 0) {
                            if (answerActions[1][0].getValue() == Action.RESIGN) return endGame(playType, RESULT_WIN1);
                        }
                    }
                } else if (playType == PLAYTYPE_REPLAY) {
                    // リプレイ中なら，棋譜から x,r を読んでその位置に変える
                    // -1 の場合そこで終了
                    for (int pl = 0; pl < 2; pl++) {
                        if (boards[pl].getChainCount() == 0) {
                            Point p = fSGFManager.getActionAsPoint(fState.getTurn(), pl);
                            if (p.X == -1) return endGame(playType, RESULT_STOPPED);
                            answerActions[pl][0] = new Action (answerActions[pl][0], p.X, p.Y);
                        }
                    }
                } else {
                    // サーバの場合は，必要なほうにgenmoveを投げる．両方そろったら必要なほうに play を投げる
                    bool[] actionRequired = new bool[2] { true, true };
                    for (int pl = 0; pl < 2; pl++) if (boards[pl].getChainCount() > 0) actionRequired[pl] = false;
                    fVSServer.askActions(actionRequired,answerActions);

                    // STOP/RESIGNが投げられたら終わる
                    for (int pl = 0; pl < 2; pl++) {
                        if (boards[pl].getChainCount() == 0) {
                            if (answerActions[pl][0].getValue() == Action.STOP) return endGame(playType, RESULT_STOPPED);
                            if (answerActions[pl][0].getValue() == Action.RESIGN) return endGame(playType, 2 - pl);
                        }
                    }
                }


                if (playType != PLAYTYPE_REPLAY) {
                    // 行動が得られたら，board.fNextColors[0],[1] を空にする
                    boards[0].deleteCurrentColors();
                    boards[1].deleteCurrentColors();

                    // 棋譜保存
                    fSGFManager.setActions(fState.getTurn(), answerActions[0][0], answerActions[1][0]);
                }

                /// Phase 4. 落下させ，消える場合は消す
                // 4-1 行動の取得と修正，表示
                for (int pl = 0; pl < 2; pl++) {
                    // 連鎖中のプレイヤは関係ない
                    if (boards[pl].getChainCount() > 0) continue;

                    // それが落下可能なのか確認し，無理なら別の場所[0]にする
                    if (answerActions[pl][0].isLegal() == false) answerActions[pl][0] = boards[pl].getPossibleActions()[0];

                    // 描画する
                    fPainters[pl].showAction(answerActions[pl][0]);
                }
                Thread.Sleep(Settings.TOTALWAIT * Settings.SLEEPAFTER_DRAWACTION);

                // 4-2 配珠ないし，浮いてる珠を落とす．消す場所も決める．
                for (int pl = 0; pl < 2; pl++) {
                    List<Point> dropStones;
                    if (boards[pl].getChainCount() == 0) {
                        // 配珠を落とす描画，実際の処理
                        dropStones = boards[pl].transit1_ActionDrop(answerActions[pl][0]);
                        fPainters[pl].drawDrop(answerActions[pl][0]);
                        // 本当はここで死ぬ判定できるのだが，まとまっていたほうが分かりやすいのでターン初めに邪魔珠落ちと同じタイミングですることにする
                    } else {
                        // 浮いてる珠（ないかもしれない）を落とす描画，実際の処理
                        dropStones = boards[pl].transit1_ChainDrop();
                        fPainters[pl].showBoard();
                    }
                    // 落ちた珠周辺を見て，消す場所を決める．
                    boards[pl].transit2_calcVanishPoints(dropStones);
                }
                Thread.Sleep(Settings.TOTALWAIT * Settings.SLEEPAFTER_DROP);


                // 4-3 連鎖による邪魔珠関係の処理
                for (int pl = 0; pl < 2; pl++) {
                    int chainCount = boards[pl].getChainCount();
                    int vanishColorStonesCount = boards[pl].getVanishColorStonesCount();

                    if (vanishColorStonesCount == 0) {
                        // 消されていない場合，連鎖が終わったら Jamの落ちるターンを決定する
                        if (chainCount > 0) {
                            boards[pl].setChainCount(0);
                            boards[1 - pl].getJamInfo().setJamTurn(fState.getTurn()+2);
                        }
                    } else {
                        // 消された場合
                        fState.asynchronize();
                        // 連鎖数を伸ばす
                        boards[pl].setChainCount(chainCount + 1);
                        // 今回得られた邪魔珠を計算し，自分にあれば相殺したあと，相手に送る
                        int gainJamCount = JamInfo.getGainJamCount(chainCount+1, vanishColorStonesCount);
                        int sendJamCount = boards[pl].getJamInfo().deleteJam(gainJamCount);
                        boards[1 - pl].getJamInfo().addJam(sendJamCount);
                    }
                }
                for (int pl = 0; pl < 2; pl++) {
                    fPainters[pl].showJamsStock();
                }

                // 4-4 実際に消す
                if (boards[0].getVanishColorStonesCount() + boards[1].getVanishColorStonesCount() > 0) {
                    for (int pl = 0; pl < 2; pl++) {
                        fPainters[pl].drawVanish_showChain(boards[pl].getChainCount());
                    }
                    Thread.Sleep(Settings.TOTALWAIT * Settings.SLEEPAFTER_SHOWCHAIN);
                    for (int pl = 0; pl < 2; pl++) {
                        fPainters[pl].drawVanish_delete();
                    }
                    Thread.Sleep(Settings.TOTALWAIT * Settings.SLEEPAFTER_VANISH);

                    for (int pl = 0; pl < 2; pl++) {
                        boards[pl].transit3_Vanish();
                    }
                }
            }
        }

        // ゲーム終了時の処理．各プレイヤへの告知や棋譜保存
        private int endGame(int playType, int resultValue) {
            if (playType == PLAYTYPE_REPLAY) {
                // 再生時は終了しても何もしない（たぶん）
                Logger.log("Replay End");
                return resultValue;
            }

            Logger.log(RESULTSTRING[resultValue]);

            // 通常時，サーバ時は棋譜を保存する
            fSGFManager.endGame(fState.getTurn(), fState.getBoard(0).getNextColors(), fState.getBoard(1).getNextColors());
            fSGFManager.saveToFile();

            if (playType == PLAYTYPE_SERVER) {
                // サーバ時は，コネクション切る
                fVSServer.closeConnection();
                Logger.log("Connection closed");
            } else {
                // 通常時は結果をプレイヤに通知する
                fPlayers[0].endGame(resultValue);
                fPlayers[1].endGame(resultValue);
                // PTPならコネクションが切れるのを待つ
                if (fIsPTP[1] == true) {
                    ((Player_PTP)fPlayers[1]).waitForBeingClosed();
                }
            }

            return resultValue;
        }

    }
}
